#!/usr/bin/env node

import TD from "turndown";
import docs from "./docs.js";
import fetch from "cross-fetch";
import fs from "fs";
import path from "path";
import iq from "inquirer";
import autocomplete from "inquirer-checkbox-autocomplete-prompt";

const SANITIZE_RE = /[/\\?%*:|"<>]/g;
const OLD_CWD = process.cwd();

iq.registerPrompt("checkbox-autocomplete", autocomplete);

iq.prompt([
  {
    type: "checkbox-autocomplete",
    name: "download",
    message: "Which documentations would you like to download?",
    choices: docs.map((i) => ({
      name: i.name + (i.version ? " - " + i.version : ""),
      value: i.slug,
    })),
  },
]).then(({ download }) => {
  for (let doc of download) {
    console.log(`Downloading: ${doc}`);
    download(doc);
  }
});

async function download(id) {
  if (fs.existsSync(id)) {
    fs.rmdirSync(id, { recursive: true, force: true });
  }
  console.log("Getting docs index");
  const folder = id;
  let idx = await fetch(
    `https://devdocs.io/docs/${id}/index.json?${Date.now()}`
  )
    .then((r) => r.json())
    .then((r) => r.entries);
  // idx = idx.slice(-3);
  fs.mkdirSync(folder, { recursive: true });
  process.chdir(folder);
  console.log("Downloading files...");
  let a = 1;
  for (let i of idx) {
    console.log(`Downloading page ${a++} of ${idx.length}`);
    await dlDoc(i, id);
  }
  process.chdir(OLD_CWD);
}

async function dlDoc(doc, id) {
  // Remove hash
  doc.path = doc.path.split("#")[0];

  const filename = j(process.cwd(), `${doc.path}.md`);
  ensureDirectoryExistence(filename);
  if (fs.existsSync(filename)) {
    return console.log("Exists");
  }

  // Hash also needs to be removed here
  const html = await fetch(
    `https://documents.devdocs.io/${id}/${doc.path}.html`
  ).then((r) => r.text());
  const converted = await html2md(html, {
    language: id,
    doc,
  });
  fs.writeFileSync(filename, converted);
}

function ensureDirectoryExistence(filePath) {
  var dirname = path.dirname(filePath);
  if (fs.existsSync(dirname)) {
    return true;
  }
  ensureDirectoryExistence(dirname);
  fs.mkdirSync(dirname, { recursive: true });
}

function html2md(html, opts) {
  const s = new TD({
    codeBlockStyle: "fenced",
    headingStyle: "atx",
    ...opts,
  });

  s.addRule("attr", {
    filter(el) {
      return el.classList.contains("_attribution") && el.tagName === "DIV";
    },
    replacement(content, el) {
      return `<!-- Attribution:\n\n${content.trim()}\n-->`;
    },
  });

  s.addRule("code-blocks", {
    filter(el) {
      return (
        el.tagName === "PRE" &&
        !el.querySelector("code") &&
        el.getAttribute("data-language")
      );
    },
    replacement: (content, el) => {
      content = content.replace("```", "\\`\\`\\`");
      return `\`\`\`${el.getAttribute("data-language")}\n${content}\`\`\``;
    },
  });

  s.addRule("links", {
    filter(node, opts) {
      return node.tagName === "A" && !/^https?:\/\//i.test(node.href);
    },
    replacement: (content, node, { language }) => {
      let h = node.href.split("#")[0];
      // for links like /advanced/info
      const docs_folder = j(OLD_CWD, language);
      if (h === "/") {
        h = "/index.md";
      }
      if (h.startsWith("/")) {
        h = path.join(docs_folder, h);
      }
      if (h === docs_folder) {
        h = j(docs_folder, "index.md");
      }
      if (!h.endsWith(".md")) {
        h += ".md";
      }
      // h = path.relative(h, docs_folder);
      if (node.hash) {
        h += node.hash;
      }
      return `[[${h}|${content}]]`;
    },
  });

  return s.turndown(html);
}

function j(...paths) {
  return path.resolve(...paths);
}
